[Libreoffice-commits] core.git: Branch 'distro/collabora/cp-6.0' - sfx2/source
Muhammet Kara (via logerrit)
logerrit at kemper.freedesktop.org
Wed Mar 27 20:40:18 UTC 2019
sfx2/source/doc/objserv.cxx | 76 +++++++++++++++++++++++++++-----------------
1 file changed, 47 insertions(+), 29 deletions(-)
New commits:
commit 91b9632395a191264b13b2d51fc6510ac9a1134c
Author: Muhammet Kara <muhammet.kara at collabora.com>
AuthorDate: Tue Mar 26 20:06:37 2019 +0300
Commit: Andras Timar <andras.timar at collabora.com>
CommitDate: Wed Mar 27 21:39:46 2019 +0100
Redaction: Handle multiple Writer pages
* Creates one GDIMetaFile from each Writer page * Inserts them as Shapes into the newly created Draw doc
* Has some 'magic' numbers needs to be figured out
* Tried with a 100-page Writer doc, seems okay
* Seems slow, might need to be optimized
* Seems like getDocumentSizeInPixels() and getDocumentSizeIn100mm()
are consuming most of the cycles. Will look into this at the end.
Change-Id: Ic6046af0c12e26185edf59862e55b17117495922
Reviewed-on: https://gerrit.libreoffice.org/65847
Tested-by: Jenkins
Reviewed-by: Muhammet Kara <muhammet.kara at collabora.com>
Reviewed-on: https://gerrit.libreoffice.org/69769
Reviewed-by: Andras Timar <andras.timar at collabora.com>
Tested-by: Andras Timar <andras.timar at collabora.com>
diff --git a/sfx2/source/doc/objserv.cxx b/sfx2/source/doc/objserv.cxx
index 4102bebe3435..43538cf7665c 100644
--- a/sfx2/source/doc/objserv.cxx
+++ b/sfx2/source/doc/objserv.cxx
@@ -542,28 +542,36 @@ void SfxObjectShell::ExecFile_Impl(SfxRequest &rReq)
if(!xCursor.is())
return;
+ xCursor->jumpToLastPage();
+ sal_Int16 nPages = xCursor->getPage();
+
uno::Reference< lang::XComponent > xSourceDoc( xModel );
DocumentToGraphicRenderer aRenderer(xSourceDoc, /*bSelectionOnly=*/false);
- // Only take the first page for now
- sal_Int16 nPage = 1;
-
- ::Size aDocumentSizePixel = aRenderer.getDocumentSizeInPixels(nPage);
- ::Point aLogicPos;
- ::Size aLogic = aRenderer.getDocumentSizeIn100mm(nPage, &aLogicPos);
- // FIXME: This is a temporary hack. Need to figure out a proper way to derive this scale factor.
- ::Size aTargetSize(aDocumentSizePixel.Width() * 1.23, aDocumentSizePixel.Height() * 1.23);
+ std::vector< GDIMetaFile > aMetaFiles;
- Graphic aGraphic = aRenderer.renderToGraphic(nPage, aDocumentSizePixel, aTargetSize, COL_TRANSPARENT);
- auto& rGDIMetaFile = const_cast<GDIMetaFile&>(aGraphic.GetGDIMetaFile());
-
- // Set preferred map unit and size on the metafile, so the Shape size
- // will be correct in MM.
- MapMode aMapMode;
- aMapMode.SetMapUnit(MapUnit::Map100thMM);
- aMapMode.SetOrigin(::Point(-aLogicPos.getX(), -aLogicPos.getY()));
- rGDIMetaFile.SetPrefMapMode(aMapMode);
- rGDIMetaFile.SetPrefSize(aLogic);
+ for (sal_Int32 nPage = 1; nPage <= nPages; ++nPage)
+ {
+ ::Size aDocumentSizePixel = aRenderer.getDocumentSizeInPixels(nPage);
+ ::Point aLogicPos;
+ ::Size aLogic = aRenderer.getDocumentSizeIn100mm(nPage, &aLogicPos);
+ // FIXME: This is a temporary hack. Need to figure out a proper way to derive this scale factor.
+ ::Size aTargetSize(aDocumentSizePixel.Width() * 1.23, aDocumentSizePixel.Height() * 1.23);
+
+ Graphic aGraphic = aRenderer.renderToGraphic(nPage, aDocumentSizePixel, aTargetSize, COL_TRANSPARENT);
+ auto& rGDIMetaFile = const_cast<GDIMetaFile&>(aGraphic.GetGDIMetaFile());
+
+ // Set preferred map unit and size on the metafile, so the Shape size
+ // will be correct in MM.
+ MapMode aMapMode;
+ aMapMode.SetMapUnit(MapUnit::Map100thMM);
+ // FIXME: This is a temporary hack. Need to figure out a proper way to derive these magic numbers.
+ aMapMode.SetOrigin(::Point(-(aLogicPos.getX() - 512) * 1.53, -((aLogicPos.getY() - 501)* 1.53 + (nPage-1)*740 )));
+ rGDIMetaFile.SetPrefMapMode(aMapMode);
+ rGDIMetaFile.SetPrefSize(aLogic);
+
+ aMetaFiles.push_back(rGDIMetaFile);
+ }
// Create an empty Draw component.
uno::Reference<frame::XDesktop2> xDesktop = css::frame::Desktop::create(comphelper::getProcessComponentContext());
@@ -573,21 +581,31 @@ void SfxObjectShell::ExecFile_Impl(SfxRequest &rReq)
// Access the draw pages
uno::Reference<drawing::XDrawPagesSupplier> xDrawPagesSupplier(xComponent, uno::UNO_QUERY);
uno::Reference<drawing::XDrawPages> xDrawPages = xDrawPagesSupplier->getDrawPages();
- uno::Reference< drawing::XDrawPage > xPage( xDrawPages->getByIndex( 0 ), uno::UNO_QUERY_THROW );
+ uno::Reference<css::lang::XMultiServiceFactory> xFactory(xComponent, uno::UNO_QUERY);
- uno::Reference<graphic::XGraphic> xGraph = aGraphic.GetXGraphic();
+ for (sal_Int32 nPage = 0; nPage < nPages; ++nPage)
+ {
+ GDIMetaFile rGDIMetaFile = aMetaFiles[nPage];
+ Graphic aGraphic(rGDIMetaFile);
- // Create and insert the shape
- uno::Reference<css::lang::XMultiServiceFactory> xFactory(xComponent, uno::UNO_QUERY);
- uno::Reference<drawing::XShape> xShape(
- xFactory->createInstance("com.sun.star.drawing.GraphicObjectShape"), uno::UNO_QUERY);
- uno::Reference<beans::XPropertySet> xShapeProperySet(xShape, uno::UNO_QUERY);
- xShapeProperySet->setPropertyValue("Graphic", uno::Any( xGraph ));
+ uno::Reference<graphic::XGraphic> xGraph = aGraphic.GetXGraphic();
+ uno::Reference< drawing::XDrawPage > xPage = xDrawPages->insertNewByIndex(nPage);
+
+ // Create and insert the shape
+ uno::Reference<drawing::XShape> xShape(
+ xFactory->createInstance("com.sun.star.drawing.GraphicObjectShape"), uno::UNO_QUERY);
+ uno::Reference<beans::XPropertySet> xShapeProperySet(xShape, uno::UNO_QUERY);
+ xShapeProperySet->setPropertyValue("Graphic", uno::Any( xGraph ));
- // Set size and position
- xShape->setSize(awt::Size(rGDIMetaFile.GetPrefSize().Width(),rGDIMetaFile.GetPrefSize().Height()) );
+ // Set size and position
+ xShape->setSize(awt::Size(rGDIMetaFile.GetPrefSize().Width(),rGDIMetaFile.GetPrefSize().Height()) );
- xPage->add(xShape);
+ xPage->add(xShape);
+ }
+
+ // Remove the extra page at the beginning
+ uno::Reference< drawing::XDrawPage > xPage( xDrawPages->getByIndex( 0 ), uno::UNO_QUERY_THROW );
+ xDrawPages->remove( xPage );
return;
}
More information about the Libreoffice-commits
mailing list